
;**************************************************************************
;*                            Feuchtesensor                               *
;*                          HYT939 / reichelt                             *
;**************************************************************************
;**************************************************************************
;*                      v 3.01 - 28.09.2018                               *
;**************************************************************************
;*          Hardw.: PIC16F1828 used                                       *
;*          OSC.......: Int. 16MHz             POWER.....:   5V extern.   *
;**************************************************************************

;release 2.01 -- calculation of humidity for HYT939 implemented
;release 3.01 -- sensor HDC1080 added

;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------

 INCLUDE "P16F1828.INC"

;-----------------------------------------------------------------------------------------
_U001			     EQU     B'00100110001010'		;see Page 48 of Datasheet

	;HS oscillator (bit2-0 = 010)
	;MCLR disabled
	;WDT enabled and controlled by SWDTEN bit of register WDTCON (bit4-3 = 01)
	;no protection
	;BOR disabled


_U002			     EQU     B'01100011111111'		;see Page 49 of Datasheet

	;write protection off
	;PLL enabled

_U003			     EQU     B'00100110000100'		;see Page 48 of Datasheet
_U004			     EQU     B'01100011111111'		;see Page 49 of Datasheet

;-----------------------------------------------------------------------------------------

	__CONFIG	_CONFIG1, _U003
	__CONFIG	_CONFIG2, _U004


;---------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-------------------------------------------------XX
#define		HYT_I2C_ADDR	H'50'		 ;XX
;-------------------------------------------------XX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;---------------------------------------------------


;==========================================================================
;       Variable Definition
;==========================================================================
TIMER1		EQU	H'20'		;delay routine
TIMER2		EQU	H'21'		; "	"	"
v_rF_MSB	EQU	H'22'
v_rF_LSB	EQU	H'23'
v_bodka_cnt	EQU	H'24'
v_bodka_flg	EQU	H'25'

v_hyt939_f_msb	EQU	H'26'
v_hyt939_f_lsb	EQU	H'27'
v_hyt939_t_msb	EQU	H'28'
v_hyt939_t_lsb	EQU	H'29'

v_rF_msb	EQU	H'2A'
v_rF_lsb	EQU	H'2B'

v_tst1		EQU	H'2C'
v_tst2		EQU	H'2D'
v_tst3		EQU	H'2E'

kbd_buffer	EQU	H'2F'

v_PRODH		EQU	H'30'
v_PRODL		EQU	H'31'

v_hyt_rF_lsb	EQU	H'32'
v_hyt_rF_msb	EQU	H'33'

v_cnt		EQU	H'34'

v_DID_h		EQU	H'35'
v_DID_l		EQU	H'36'

v_device	EQU	H'37'		;bit 0 = 1 -> expected device is HYT939
					;bit 1 = 1 -> expected device is HDC1080
;--------------------------------------------------------------------------
;==========================================================================

		ORG	0		;Reset vector address
		GOTO	init_main	;goto RESET routine when boot.

;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;
;	       **********************************
;              **  RESET :  main boot routine  **
;              **********************************
;
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
; initialization
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

init_main	nop

		banksel	OSCCON
		movf	OSCCON,0
		iorlw	B'01111000'
		movwf	OSCCON			;set 16 MHz internal clock (bit 6:3 = 1111) - Datasheet Page 68

		clrf	BSR

		clrf	INTCON			;disable interrupts

		movlw	H'FF'
		movwf	PORTB

		banksel	OPTION_REG		;BANK 1
		movlw	B'01111111'
		movwf	OPTION_REG		;see datasheet Page 176

		movlw	H'03'
		movwf	BSR			;select BANK 3
		clrf	ANSELA
		clrf	ANSELB
		clrf	ANSELC

		movlw	H'01'
		movwf	BSR			;select BANK 1

		movlw	B'11001000'
		movwf	TRISA

		movlw	B'01011111'
		movwf	TRISB

		movlw	B'00000000'
		movwf	TRISC

;no changes required to OSCCON as after RESET the value from Config 1 word will be used (OSCCON - bit 1:bit 0 = 00)

		movlw	H'04'
		movwf	BSR			;select BANK 4
		clrf	WPUA			;disable weak pull-ups for PORTA
		clrf	WPUB			;disable weak pull-ups for PORTB
		clrf	WPUC			;disable weak pull-ups for PORTC

		movlw	H'07'
		movwf	BSR			;select BANK 7
		movlw	H'FF'
		movwf	INLVLA			;ST (1) level or TTL (0) level (?)
		movwf	INLVLB			;ST (1) level or TTL (0) level (?)
		movwf	INLVLC			;ST (1) level or TTL (0) level (?)

		banksel	LATC
		clrf	LATA
		clrf	LATB
		clrf	LATC

		clrf	BSR			;select BANK 0

;comment: no need to change the WDTCON register
;- as reset-value set to WDT to 2 seconds - what is ok for the application
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;		banksel	WDTCON
;		bsf	WDTCON,D'000'		;enable Watch-dog
;		clrf	BSR			;select BANK 0
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

		call	i2c_init

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      Main Program
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

;		call	disp_test
		clrf	v_bodka_cnt

		call	read_DID		;Device ID for HDC1080 is retrieved
		call	set_device		;based on this, decision about connected sensor is taken
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
main_prg	incf	v_bodka_cnt,1
		movlw	D'128'
		movwf	v_bodka_flg
		btfss	v_bodka_cnt,D'002'
		clrf	v_bodka_flg
;---------------------------------------------
		clrf	v_hyt_rF_lsb
		clrf	v_hyt_rF_msb
;
		btfsc	v_device,D'000'
		call	device_1_exec
		btfsc	v_device,D'001'
		call	device_2_exec
;---------------------------------------------
		call	w_hyt939_hum
;---------------------------------------------
		goto	main_prg
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
device_1_exec	call	hyt939_start
		call	dr2
		call	hyt939_read
		call	hyt939_convert
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
device_2_exec	call	hdc1080_humi
		call	hdc1080_conv
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
w_hyt939_hum	call	w_hyt939_hum_l
		call	w_hyt939_hum_m
		return
;-------------------------------------------
w_hyt939_hum_l	movf	v_hyt_rF_lsb,0
		brw
		goto	w_D2_0
		goto	w_D2_1
		goto	w_D2_2
		goto	w_D2_3
		goto	w_D2_4
		goto	w_D2_5
		goto	w_D2_6
		goto	w_D2_7
		goto	w_D2_8
		goto	w_D2_9
		goto	w_D2_minus
;-------------------------------------------
w_hyt939_hum_m	movf	v_hyt_rF_msb,0
		brw
		goto	w_D1_0
		goto	w_D1_1
		goto	w_D1_2
		goto	w_D1_3
		goto	w_D1_4
		goto	w_D1_5
		goto	w_D1_6
		goto	w_D1_7
		goto	w_D1_8
		goto	w_D1_9
		goto	w_D1_minus
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
disp_test	call	w_D1_0
		call	dr2
		call	w_D1_1
		call	dr2
		call	w_D1_2
		call	dr2
		call	w_D1_3
		call	dr2
		call	w_D1_4
		call	dr2
		call	w_D1_5
		call	dr2
		call	w_D1_6
		call	dr2
		call	w_D1_7
		call	dr2
		call	w_D1_8
		call	dr2
		call	w_D1_9
		call	dr2
;----------------------------------------
		call	w_D1_space
;----------------------------------------
		call	w_D2_0
		call	dr2
		call	w_D2_1
		call	dr2
		call	w_D2_2
		call	dr2
		call	w_D2_3
		call	dr2
		call	w_D2_4
		call	dr2
		call	w_D2_5
		call	dr2
		call	w_D2_6
		call	dr2
		call	w_D2_7
		call	dr2
		call	w_D2_8
		call	dr2
		call	w_D2_9
		call	dr2
;----------------------------------------
		call	w_D2_space
;----------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
w_D2_space	clrf	PORTC
		return
;---------------------------------------
w_D2_0		movlw	B'00111111'
		addwf	v_bodka_flg,0
		movwf	PORTC
		return
;---------------------------------------
w_D2_1		movlw	B'00000110'
		addwf	v_bodka_flg,0
		movwf	PORTC
		return
;---------------------------------------
w_D2_2		movlw	B'01011011'
		addwf	v_bodka_flg,0
		movwf	PORTC
		return
;---------------------------------------
w_D2_3		movlw	B'01001111'
		addwf	v_bodka_flg,0
		movwf	PORTC
		return
;---------------------------------------
w_D2_4		movlw	B'01100110'
		addwf	v_bodka_flg,0
		movwf	PORTC
		return
;---------------------------------------
w_D2_5		movlw	B'01101101'
		addwf	v_bodka_flg,0
		movwf	PORTC
		return
;---------------------------------------
w_D2_6		movlw	B'01111101'
		addwf	v_bodka_flg,0
		movwf	PORTC
		return
;---------------------------------------
w_D2_7		movlw	B'00100111'
		addwf	v_bodka_flg,0
		movwf	PORTC
		return
;---------------------------------------
w_D2_8		movlw	B'01111111'
		addwf	v_bodka_flg,0
		movwf	PORTC
		return
;---------------------------------------
w_D2_9		movlw	B'01101111'
		addwf	v_bodka_flg,0
		movwf	PORTC
		return
;---------------------------------------
w_D2_minus	movlw	B'01000000'
		addwf	v_bodka_flg,0
		movwf	PORTC
		return
;------------------------------------------------------------------------------
w_D1_space	nop
		banksel	LATA
		bcf	LATA,D'000'
		bcf	LATA,D'001'
		bcf	LATA,D'002'
		bcf	LATA,D'004'
		bcf	LATA,D'005'
		bcf	LATB,D'005'
		bcf	LATB,D'007'
		clrf	BSR
		return
;---------------------------------------
w_D1_0		nop
		banksel	LATA
		bsf	LATA,D'000'
		bsf	LATA,D'001'
		bsf	LATA,D'002'
		bsf	LATA,D'004'
		bsf	LATA,D'005'
		bsf	LATB,D'005'
		bcf	LATB,D'007'
		clrf	BSR
		return
;---------------------------------------
w_D1_1		nop
		banksel	LATA
		bcf	LATA,D'000'
		bsf	LATA,D'001'
		bsf	LATA,D'002'
		bcf	LATA,D'004'
		bcf	LATA,D'005'
		bcf	LATB,D'005'
		bcf	LATB,D'007'
		clrf	BSR
		return
;---------------------------------------
w_D1_2		nop
		banksel	LATA
		bsf	LATA,D'000'
		bsf	LATA,D'001'
		bcf	LATA,D'002'
		bsf	LATA,D'004'
		bsf	LATA,D'005'
		bcf	LATB,D'005'
		bsf	LATB,D'007'
		clrf	BSR
		return
;---------------------------------------
w_D1_3		nop
		banksel	LATA
		bsf	LATA,D'000'
		bsf	LATA,D'001'
		bsf	LATA,D'002'
		bsf	LATA,D'004'
		bcf	LATA,D'005'
		bcf	LATB,D'005'
		bsf	LATB,D'007'
		clrf	BSR
		return
;---------------------------------------
w_D1_4		nop
		banksel	LATA
		bcf	LATA,D'000'
		bsf	LATA,D'001'
		bsf	LATA,D'002'
		bcf	LATA,D'004'
		bcf	LATA,D'005'
		bsf	LATB,D'005'
		bsf	LATB,D'007'
		clrf	BSR
		return
;---------------------------------------
w_D1_5		nop
		banksel	LATA
		bsf	LATA,D'000'
		bcf	LATA,D'001'
		bsf	LATA,D'002'
		bsf	LATA,D'004'
		bcf	LATA,D'005'
		bsf	LATB,D'005'
		bsf	LATB,D'007'
		clrf	BSR
		return
;---------------------------------------
w_D1_6		nop
		banksel	LATA
		bsf	LATA,D'000'
		bcf	LATA,D'001'
		bsf	LATA,D'002'
		bsf	LATA,D'004'
		bsf	LATA,D'005'
		bsf	LATB,D'005'
		bsf	LATB,D'007'
		clrf	BSR
		return
;---------------------------------------
w_D1_7		nop
		banksel	LATA	
		bsf	LATA,D'000'
		bsf	LATA,D'001'
		bsf	LATA,D'002'
		bcf	LATA,D'004'
		bcf	LATA,D'005'
		bsf	LATB,D'005'
		bcf	LATB,D'007'
		clrf	BSR
		return
;---------------------------------------
w_D1_8		nop
		banksel	LATA
		bsf	LATA,D'000'
		bsf	LATA,D'001'
		bsf	LATA,D'002'
		bsf	LATA,D'004'
		bsf	LATA,D'005'
		bsf	LATB,D'005'
		bsf	LATB,D'007'
		clrf	BSR
		return
;---------------------------------------
w_D1_9		nop
		banksel	LATA
		bsf	LATA,D'000'
		bsf	LATA,D'001'
		bsf	LATA,D'002'
		bsf	LATA,D'004'
		bcf	LATA,D'005'
		bsf	LATB,D'005'
		bsf	LATB,D'007'
		clrf	BSR
		return
;---------------------------------------
w_D1_minus	nop
		banksel	LATA
		bcf	LATA,D'000'
		bcf	LATA,D'001'
		bcf	LATA,D'002'
		bcf	LATA,D'004'
		bcf	LATA,D'005'
		bcf	LATB,D'005'
		bsf	LATB,D'007'
		clrf	BSR
		return
;---------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
; delay routines
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
d2		MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOP1	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOP2	DECFSZ  TIMER1,F
		GOTO    DEL_LOOP2
		DECFSZ  TIMER2,F
		GOTO    DEL_LOOP1
		return
;------------------------------------------------------------------------------
d2c		MOVLW   D'090'
		MOVWF   TIMER2
cDEL_LOOP1	MOVLW   D'255'
		MOVWF   TIMER1

cDEL_LOOP2	DECFSZ  TIMER1,F
		GOTO    cDEL_LOOP2
		DECFSZ  TIMER2,F
		GOTO    cDEL_LOOP1
		return
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

;------------------------------------
dr2		call	d2
		call	d2
		call	d2
		return
;------------------------------------
dr4		call	dr2
		call	dr2
		call	dr2
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
 INCLUDE "90_I2C_comm_PIC16F1829.inc"
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;HDC1080 communication routines
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
hdc1080_humi	call	i2c_start
		movlw	H'80'
		call	i2c_send
		movlw	H'01'
		call	i2c_send
		call	i2c_stop
;
		call	d2
;read humidity --------------------------
		call	i2c_start
		movlw	H'81'
		call	i2c_send

		call	i2c_receive
		movwf	v_hyt939_f_msb
		call	i2c_ack
		call	i2c_receive
		movwf	v_hyt939_f_lsb
		call	i2c_not_ack
		call	i2c_stop

		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
hdc1080_conv	call	hyt939_no_sen
		call	read_DID
		movf	v_DID_h,0
		xorlw	H'FF'
		btfsc	STATUS,Z
		goto	hyt939_no_sen

;convert humidity ------------------------------
		movf	v_hyt939_f_msb,0
		movwf	v_rF_msb
		movf	v_hyt939_f_lsb,0
		movwf	v_rF_lsb
;------------------------------------------------
;the 8-Bit meassured value of humidity in v_rF_msb
;now we can multiply by 100 (derived information from datasheet)
;		movf	v_rF_msb,0
;		mullw	D'100'
;for your microcontroller we have to do the multiplication "manually" ------------
		clrf	v_PRODH
		clrf	v_PRODL

		movlw	D'100'
		movwf	v_cnt

convert2_loop1	movf	v_rF_msb,0
		addwf	v_PRODL,1
		btfsc	STATUS,C
		incf	v_PRODH,1

		decfsz	v_cnt,1
		goto	convert2_loop1
;---------------------------------------------------------------------------------
;the result is placed in PRODH:PRODL
;now we need to divide the result by 256 -> means, we have the result alread in PRODH
		movf	v_PRODH,0
		movwf	v_rF_msb
		movf	v_PRODL,0
		movwf	v_rF_lsb

		movf	v_rF_msb,0
		movwf	kbd_buffer
		call	write_dec_dummy
		movf	v_tst3,0
		movwf	v_hyt_rF_lsb
		movf	v_tst2,0
		movwf	v_hyt_rF_msb
;--------------------------------------------------
;the rF is now stored in v_rF
;--------------------------------------------------

		movlw	'0'
		subwf	v_hyt_rF_lsb,1
		subwf	v_hyt_rF_msb,1

		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------




;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;HYT939 communication routines
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
hyt939_start	call	i2c_start
		movlw	HYT_I2C_ADDR
		call	i2c_send
		call	i2c_stop
		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
hyt939_read	call	i2c_start
		movlw	HYT_I2C_ADDR
		addlw	D'001'			;read
		call	i2c_send
;read humidity --------------------------
		call	i2c_receive
		movwf	v_hyt939_f_msb
		call	i2c_ack
		call	i2c_receive
		movwf	v_hyt939_f_lsb
		call	i2c_ack
;read temperature -----------------------
		call	i2c_receive
		movwf	v_hyt939_t_msb
		call	i2c_ack
		call	i2c_receive
		andlw	B'11111100'		;mask the first two LSb (not used bits)
		movwf	v_hyt939_t_lsb
;----------------------------------------
		call	i2c_not_ack
		call	i2c_stop

		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
hyt939_no_sen	movlw	H'0A'
		movwf	v_hyt_rF_msb
		movwf	v_hyt_rF_lsb
		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
hyt939_convert	nop
		movf	v_hyt939_t_msb,0
		xorlw	H'FF'
		btfsc	STATUS,Z
		goto	hyt939_no_sen

;convert humidity ------------------------------
		movf	v_hyt939_f_msb,0
		andlw	B'00111111'		;mask the first two MSb (status bits)
		movwf	v_rF_msb
		movf	v_hyt939_f_lsb,0
		movwf	v_rF_lsb
;------------------------------------------------
;now we put the two MSb from v_rF_lsb to v_rF_msb --> 8-bit resolution (isntead of 6 bit in v_rF_msb after reading)
		rlf	v_rF_lsb,1
		rlf	v_rF_msb,1
		rlf	v_rF_lsb,1
		rlf	v_rF_msb,1
;------------------------------------------------
;now we have the 8-Bit meassured value of humidity in v_rF_msb
;now we can multiply by 25 (derived information from datasheet)
;		movf	v_rF_msb,0		;for PIC18
;		mullw	D'025'			;for PIC18
;for your microcontroller we have to do the multiplication "manually" ------------
		clrf	v_PRODH
		clrf	v_PRODL

		movlw	D'025'
		movwf	v_cnt

convert_loop1	movf	v_rF_msb,0
		addwf	v_PRODL,1
		btfsc	STATUS,C
		incf	v_PRODH,1

		decfsz	v_cnt,1
		goto	convert_loop1
;---------------------------------------------------------------------------------
;the result is placed in PRODH:PRODL
;now we need to divide the result by 64
		movf	v_PRODH,0
		movwf	v_rF_msb
		movf	v_PRODL,0
		movwf	v_rF_lsb

		rrf	v_rF_msb,1
		rrf	v_rF_lsb,1
		rrf	v_rF_msb,1
		rrf	v_rF_lsb,1
		rrf	v_rF_msb,1
		rrf	v_rF_lsb,1
		rrf	v_rF_msb,1
		rrf	v_rF_lsb,1
		rrf	v_rF_msb,1
		rrf	v_rF_lsb,1
		rrf	v_rF_msb,1
		rrf	v_rF_lsb,1

		movf	v_rF_msb,0
		andlw	B'00000011'
		movwf	v_rF_msb

;at this moment - v_rF_msb have to be 00h
;               - v_rF_lsb shall be the rF

		movf	v_rF_lsb,0

		call	write_dec_dummy
		movf	v_tst3,0
		movwf	v_hyt_rF_lsb
		movf	v_tst2,0
		movwf	v_hyt_rF_msb

		movlw	'0'
		subwf	v_hyt_rF_lsb,1
		subwf	v_hyt_rF_msb,1
;--------------------------------------------------
;the rF is now stored in v_rF
;--------------------------------------------------
		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
write_dec_dummy	movwf	kbd_buffer

		movlw	'0'
		movwf	v_tst1
		movwf	v_tst2
		movwf	v_tst3

conv_next_dm	movf	kbd_buffer,0
		btfsc	STATUS,Z
		return

		incf	v_tst3,1
		movf	v_tst3,0
		xorlw	':'
		btfss	STATUS,Z
		goto	wdcn_dm

		movlw	'0'
		movwf	v_tst3
		incf	v_tst2,1
		movf	v_tst2,0
		xorlw	':'
		btfss	STATUS,Z
		goto	wdcn_dm

		movlw	'0'
		movwf	v_tst2
		incf	v_tst1,1

wdcn_dm		decf	kbd_buffer,1
		goto	conv_next_dm

		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;Receive DID (Device ID) for Sensor HDC1080
;sensor I2C Address is: 80h
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;read DID (Device ID)
read_DID	call	i2c_start
		movlw	H'80'
		call	i2c_send
		movlw	H'FF'
		call	i2c_send
		call	i2c_stop
;
		call	i2c_start
		movlw	H'81'
		call	i2c_send
		call	i2c_receive
		movwf	v_DID_h
		call	i2c_ack
		call	i2c_receive
		movwf	v_DID_l
		call	i2c_not_ack
		call	i2c_stop
		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
set_device	clrf	v_device
		movf	v_DID_h,0
		xorlw	H'10'
		btfss	STATUS,Z
		goto	device_HYT939
device_HDC1080	bsf	v_device,D'001'		;HDC1080
		return
device_HYT939	bsf	v_device,D'000'		;HYT939
		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

		END